@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License. 
 
+import os
 import json
 import yaml
 import oscar_python._utils as utils
@@ -34,6 +35,7 @@ _GET = "get"
 _POST = "post"
 _PUT = "put"
 _DELETE = "delete"
+_DEFAULT_TIMEOUT = 30
 
 class Client:
     #Cluster info 
@@ -70,57 +72,72 @@ class Client:
     def _apply_service(self, fdl_path, method):
         with open(fdl_path, "r") as read_fdl:
             fdl = self._parse_FDL_yaml(read_fdl)
+        # Read FDL file and check correct format
         if fdl != ValueError:
             for element in fdl["functions"]["oscar"]:
                 try:
                     svc = element[self.id]
                 except KeyError as err:
-                    print("Error: FDL clusterID don't match current clusterID: {0}".format(err))
-                    return err
+                    raise("FDL clusterID does not match current clusterID: {0}".format(err))
+                # Check if service already exists when the function is called from create_service
+                if method == _POST:
+                    svc_exists = utils.make_request(self, _SVC_PATH+"/"+svc["name"], _GET, handle=False)
+                    if svc_exists.status_code == 200:
+                        raise ValueError("A service with name '{0}' is already present on the cluster".format(svc["name"]))
                 try:
                     with open(svc["script"]) as s:
                         svc["script"] = s.read()
                 except IOError as err:
-                    print("Error: Couldn't read script")
-                    return err
+                    raise("Couldn't read script")
 
                 # cpu parameter has to be string on the request
                 if type(svc["cpu"]) is int or type(svc["cpu"]) is float: svc["cpu"]= str(svc["cpu"])
-                return utils.make_request(self, _SVC_PATH, method, json.dumps(svc))
+                utils.make_request(self, _SVC_PATH, method, data=json.dumps(svc))
         else:
-            print("Error: Bad yaml format: {0}".format(fdl))
-            return ValueError
+            raise ValueError("Bad yaml format: {0}".format(fdl))
 
     """ Create a service on the current cluster from a FDL file """
     def create_service(self, fdl_path):
-        self._apply_service(fdl_path, _POST)
+        return self._apply_service(fdl_path, _POST)
 
     """ Update a specific service """
     def update_service(self, name, fdl_path):
-        svc = self.get_service(name)
+        # Check if service exists before update
+        svc = utils.make_request(self, _SVC_PATH+"/"+svc["name"], _GET, handle=False)
         if svc.status_code != 200:
-            print("Error: the service {0} is not present on the cluster".format(name))
-            return
-        self._apply_service(fdl_path, _PUT)
+            raise ValueError("The service {0} is not present on the cluster".format(name))
+        return self._apply_service(fdl_path, _PUT)
 
     """ Remove a specific service """
     def remove_service(self, name):
         return utils.make_request(self, _SVC_PATH+"/"+name, _DELETE)
 
-    """ Run a synchronous execution """
-    def run_service(self, name, input=""):
-        token = self._get_token(name)
-        if input: return utils.make_request(self, _RUN_PATH+"/"+name, _POST, data=input, token=token)
+    """ Run a synchronous execution. 
+        If an output is provided the result is decoded onto the file.
+        In both cases the function returns the HTTP response."""
+    def run_service(self, name, **kwargs):
+        if "input" in kwargs.keys() and kwargs["input"]:
+            exec_input = kwargs["input"]
+            token = self._get_token(name) 
+            
+            send_data = utils.encode_input(exec_input)
+
+            if "timeout" in kwargs.keys() and kwargs["timeout"]:
+                response = utils.make_request(self, _RUN_PATH+"/"+name, _POST, data=send_data, token=token, timeout=kwargs["timeout"])
+            else:
+                response = utils.make_request(self, _RUN_PATH+"/"+name, _POST, data=send_data, token=token)
+            
+            if "output" in kwargs.keys() and kwargs["output"]:
+                utils.decode_output(response.text, kwargs["output"])
+            return response
+        
         return utils.make_request(self, _RUN_PATH+"/"+name, _POST, token=token)
     
-    """ Run an asynchronous execution (not usable at the moment). """
-    #TODO fix
+    """ Run an asynchronous execution (unable at the moment). """
+    #TODO
     """ def _run_job(self, name, input_path =""):
-        token = self._get_token(name)
-        if input:
-            files = {'input': open(input_path, "rb")}
-            return utils.make_request(self, "/job/"+name, _POST, file=files, token=token)
-        return utils.make_request(self, "job/"+name, _POST, token=token) """
+            pass 
+    """
 
     def _get_token(self, svc):
         service = utils.make_request(self, _SVC_PATH+"/"+svc, _GET)
@@ -134,14 +151,18 @@ class Client:
             return err
         return fdl_yaml
     
+    """ Get logs of a service job """
     def get_job_logs(self, svc, job):
         return utils.make_request(self, _LOGS_PATH+"/"+svc+"/"+job, _GET)
-
+    
+    """ List a service jobs """
     def list_jobs(self, svc):
         return utils.make_request(self, _LOGS_PATH+"/"+svc, _GET)
 
+    """ Remove a service job """
     def remove_job(self, svc, job):
         return utils.make_request(self, _LOGS_PATH+"/"+svc+"/"+job, _DELETE)
 
+    """ Remove all service jobs """
     def remove_all_jobs(self, svc):
         return utils.make_request(self, _LOGS_PATH+"/"+svc, _DELETE)
